import pytest
from async_generator import async_generator, yield_

from anyio import sleep


@pytest.fixture
async def async_fixture():
    await sleep(0)
    return 'foo'


@pytest.fixture
@async_generator
async def asyncgen_fixture():
    await sleep(0)
    await yield_('foo')
    await sleep(0)


@pytest.mark.anyio(backends=['asyncio'])
async def test_fixture(async_fixture):
    assert async_fixture == 'foo'


@pytest.mark.anyio
async def test_asyncgen_fixture(asyncgen_fixture):
    assert asyncgen_fixture == 'foo'



from abc import ABCMeta, abstractmethod
from io import SEEK_SET
from ipaddress import IPv4Address, IPv6Address
from ssl import SSLContext
from typing import Callable, TypeVar, Optional, Tuple, Union

T_Retval = TypeVar('T_Retval')
IPAddressType = Union[str, IPv4Address, IPv6Address]
BufferType = Union[bytes, bytearray, memoryview]


class Lock(metaclass=ABCMeta):
    @abstractmethod
    async def __aenter__(self):
        pass

    @abstractmethod
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    @abstractmethod
    def locked(self) -> bool:
        """Return True if the lock is currently held."""


class Condition(metaclass=ABCMeta):
    @abstractmethod
    async def __aenter__(self):
        pass

    @abstractmethod
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    @abstractmethod
    def locked(self) -> bool:
        """Return True if the lock is set."""

    @abstractmethod
    async def notify(self, n: int = 1) -> None:
        """Notify exactly n listeners."""

    @abstractmethod
    async def notify_all(self) -> None:
        """Notify all the listeners."""

    @abstractmethod
    async def wait(self) -> None:
        """Wait for a notification."""


class Event(metaclass=ABCMeta):
    @abstractmethod
    async def set(self) -> None:
        pass

    @abstractmethod
    def clear(self) -> None:
        pass

    @abstractmethod
    def is_set(self) -> None:
        pass

    @abstractmethod
    async def wait(self) -> bool:
        pass


class Semaphore(metaclass=ABCMeta):
    @abstractmethod
    async def __aenter__(self):
        pass

    @abstractmethod
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    @property
    @abstractmethod
    def value(self) -> int:
        pass


class Queue(metaclass=ABCMeta):
    @abstractmethod
    def empty(self) -> bool:
        pass

    @abstractmethod
    def full(self) -> bool:
        pass

    @abstractmethod
    def qsize(self) -> int:
        pass

    @abstractmethod
    async def put(self, item) -> None:
        pass

    @abstractmethod
    async def get(self):
        pass


class TaskGroup(metaclass=ABCMeta):
    cancel_scope = None  # type: CancelScope

    @abstractmethod
    async def spawn(self, func: Callable, *args, name=None) -> None:
        pass


class CancelScope(metaclass=ABCMeta):
    @abstractmethod
    async def cancel(self):
        """Cancel all tasks within this scope."""


class AsyncFile(metaclass=ABCMeta):
    """
    An asynchronous file object.

    This class wraps a standard file object and provides async friendly versions of the following
    blocking methods (where available on the original file object):

    * read
    * read1
    * readline
    * readlines
    * readinto
    * readinto1
    * write
    * writelines
    * truncate
    * seek
    * tell
    * flush
    * close

    All other methods are directly passed through.

    This class supports the asynchronous context manager protocol which closes the underlying file
    at the end of the context block.

    This class also supports asynchronous iteration::

        async with await aopen(...) as f:
            async for line in f:
                print(line)
    """

    @abstractmethod
    async def __aenter__(self):
        pass

    @abstractmethod
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    @abstractmethod
    async def __aiter__(self):
        pass

    @abstractmethod
    async def read(self, size: int = -1) -> Union[bytes, str]:
        pass

    @abstractmethod
    async def read1(self, size: int = -1) -> Union[bytes, str]:
        pass

    @abstractmethod
    async def readline(self) -> bytes:
        pass

    @abstractmethod
    async def readlines(self) -> bytes:
        pass

    @abstractmethod
    async def readinto(self, b: Union[bytes, memoryview]) -> bytes:
        pass

    @abstractmethod
    async def readinto1(self, b: Union[bytes, memoryview]) -> bytes:
        pass

    @abstractmethod
    async def write(self, b: bytes) -> None:
        pass

    @abstractmethod
    async def writelines(self, lines: bytes) -> None:
        pass

    @abstractmethod
    async def truncate(self, size: Optional[int] = None) -> int:
        pass

    @abstractmethod
    async def seek(self, offset: int, whence: Optional[int] = SEEK_SET) -> int:
        pass

    @abstractmethod
    async def tell(self) -> int:
        pass

    @abstractmethod
    async def flush(self) -> None:
        pass

    @abstractmethod
    async def close(self) -> None:
        pass


class Stream(metaclass=ABCMeta):
    @abstractmethod
    async def receive_some(self, max_bytes: Optional[int]) -> bytes:
        """
        Reads up to the given amount of bytes from the stream.

        :param max_bytes: maximum number of bytes to read
        :return: the bytes read
        """

    @abstractmethod
    async def receive_exactly(self, nbytes: int) -> bytes:
        """
        Read exactly the given amount of bytes from the stream.

        :param nbytes: the number of bytes to read
        :return: the bytes read
        :raises anyio.exceptions.IncompleteRead: if the stream was closed before the requested
            amount of bytes could be read from the stream
        """

    @abstractmethod
    async def receive_until(self, delimiter: bytes, max_bytes: int) -> bytes:
        """
        Read from the stream until the delimiter is found or max_bytes have been read.

        :param delimiter: the marker to look for in the stream
        :param max_bytes: maximum number of bytes that will be read before raising
            :exc:`~anyio.exceptions.DelimiterNotFound`
        :return: the bytes read, including the delimiter
        :raises anyio.exceptions.IncompleteRead: if the stream was closed before the delimiter
            was found
        :raises anyio.exceptions.DelimiterNotFound: if the delimiter is not found within the
            bytes read up to the maximum allowed
        """

    @abstractmethod
    async def send_all(self, data: BufferType) -> None:
        """
        Send all of the given data to the other end.

        :param data: the buffer to send
        """


class SocketStream(Stream):
    async def __aenter__(self):
        return self

    async def __aexit__(self, *exc_info):
        self.close()

    @abstractmethod
    def close(self) -> None:
        """Close the underlying socket."""

    @abstractmethod
    async def start_tls(self, context: Optional[SSLContext] = None) -> None:
        pass


class SocketStreamServer(metaclass=ABCMeta):
    async def __aenter__(self):
        return self

    async def __aexit__(self, *exc_info):
        self.close()

    @abstractmethod
    def close(self) -> None:
        """Close the underlying socket."""

    @property
    @abstractmethod
    def address(self) -> Union[Tuple[str, int], str]:
        """Return the bound address of the underlying socket."""

    @property
    def port(self) -> int:
        """
        Return the currently bound port of the underlying socket.

        Equivalent to ``server.address[1]``.
        """
        return self.address[1]

    @abstractmethod
    async def accept(self) -> SocketStream:
        """
        Accept an incoming connection.

        :return: the socket stream for the accepted connection
        """


class DatagramSocket(metaclass=ABCMeta):
    async def __aenter__(self):
        return self

    async def __aexit__(self, *exc_info):
        self.close()

    @abstractmethod
    def close(self) -> None:
        """Close the underlying socket."""

    @property
    @abstractmethod
    def address(self) -> Tuple[str, int]:
        """Return the bound address of the underlying socket."""

    @property
    def port(self) -> int:
        """
        Return the currently bound port of the underlying socket.

        Equivalent to ``socket.address[1]``.
        """
        return self.address[1]

    @abstractmethod
    async def receive(self, max_bytes: int) -> Tuple[bytes, str]:
        pass

    @abstractmethod
    async def send(self, data: BufferType, address: Optional[str] = None,
                   port: Optional[int] = None) -> None:
        pass



import curio
import pytest
import trio

from anyio import (
    create_task_group, sleep, move_on_after, fail_after, open_cancel_scope,
    reset_detected_asynclib)
from anyio._backends import asyncio
from anyio.exceptions import ExceptionGroup


async def async_error(text, delay=0.1):
    try:
        if delay:
            await sleep(delay)
    finally:
        raise Exception(text)


@pytest.mark.anyio
async def test_already_closed():
    async with create_task_group() as tg:
        pass

    with pytest.raises(RuntimeError) as exc:
        await tg.spawn(async_error, 'fail')

    exc.match('This task group is not active; no new tasks can be spawned')


@pytest.mark.anyio
async def test_success():
    async def async_add(value):
        results.add(value)

    results = set()
    async with create_task_group() as tg:
        await tg.spawn(async_add, 'a')
        await tg.spawn(async_add, 'b')

    assert results == {'a', 'b'}


@pytest.mark.parametrize('run_func, as_coro_obj', [
    (asyncio.native_run, True),
    (curio.run, False),
    (trio.run, False)
], ids=['asyncio', 'curio', 'trio'])
def test_run_natively(run_func, as_coro_obj):
    async def testfunc():
        async with create_task_group() as tg:
            await tg.spawn(sleep, 0)

    reset_detected_asynclib()
    if as_coro_obj:
        run_func(testfunc())
    else:
        run_func(testfunc)


@pytest.mark.anyio
async def test_host_exception():
    async def set_result(value):
        nonlocal result
        await sleep(3)
        result = value

    result = None
    with pytest.raises(Exception) as exc:
        async with create_task_group() as tg:
            await tg.spawn(set_result, 'a')
            raise Exception('dummy error')

    exc.match('dummy error')
    assert result is None


@pytest.mark.anyio
async def test_edge_cancellation():
    async def dummy():
        nonlocal marker
        marker = 1
        # At this point the task has been cancelled so sleep() will raise an exception
        await sleep(0)
        # Execution should never get this far
        marker = 2

    marker = None
    async with create_task_group() as tg:
        await tg.spawn(dummy)
        await tg.cancel_scope.cancel()

    assert marker == 1


# @pytest.mark.anyio
# async def test_host_cancelled_before_aexit():
#     async def set_result(value):
#         nonlocal result
#         await sleep(3)
#         result = value
#
#     async def host():
#         async with create_task_group() as tg:
#             await tg.spawn(set_result, 'a')
#
#     result = None
#     with pytest.raises(event_loop.CancelledError):
#         with open_cancel_scope() as scope:
#             async with create_task_group() as main_group:
#                 await main_group.spawn(host)
#                 await sleep(0.1)
#                 scope.cancel()
#
#     assert result is None


# @pytest.mark.anyio
# async def test_host_cancelled_during_aexit(event_loop, queue):
#     with pytest.raises(CancelledError):
#         async with event_loop.TaskGroup() as tg:
#             tg.spawn(self.delayed_put, event_loop, queue, 'a')
#             event_loop.call_soon(asyncio.Task.current_task().cancel)
#
#     assert queue.empty()


@pytest.mark.anyio
async def test_cancel_scope_in_another_task():
    async def child():
        nonlocal result, local_scope
        async with open_cancel_scope() as local_scope:
            await sleep(2)
            result = True

    local_scope = None
    result = False
    async with create_task_group() as tg:
        await tg.spawn(child)
        while local_scope is None:
            await sleep(0)

        await local_scope.cancel()

    assert not result


@pytest.mark.anyio
async def test_multi_error_children():
    with pytest.raises(ExceptionGroup) as exc:
        async with create_task_group() as tg:
            await tg.spawn(async_error, 'task1')
            await tg.spawn(async_error, 'task2')

    assert len(exc.value.exceptions) == 2
    assert sorted(str(e) for e in exc.value.exceptions) == ['task1', 'task2']


@pytest.mark.anyio
async def test_multi_error_host():
    with pytest.raises(ExceptionGroup) as exc:
        async with create_task_group() as tg:
            await tg.spawn(async_error, 'child', 2)
            await sleep(0.1)
            raise Exception('host')

    assert len(exc.value.exceptions) == 2
    assert [str(e) for e in exc.value.exceptions] == ['host', 'child']


@pytest.mark.anyio
async def test_escaping_cancelled_exception():
    async with create_task_group() as tg:
        await tg.cancel_scope.cancel()
        await sleep(0)


@pytest.mark.anyio
async def test_fail_after():
    with pytest.raises(TimeoutError):
        async with fail_after(0.1):
            await sleep(1)


@pytest.mark.anyio
async def test_fail_after_no_timeout():
    async with fail_after(None):
        await sleep(0.1)


@pytest.mark.anyio
async def test_move_on_after():
    result = False
    async with move_on_after(0.1):
        await sleep(1)
        result = True

    assert not result


@pytest.mark.anyio
async def test_move_on_after_no_timeout():
    result = False
    async with move_on_after(None):
        await sleep(0.1)
        result = True

    assert result



from pathlib import Path

import pytest

from anyio import aopen


@pytest.fixture(scope='module')
def testdata():
    return b''.join(bytes([i] * 1000) for i in range(10))


@pytest.fixture
def testdatafile(tmpdir_factory, testdata):
    file = tmpdir_factory.mktemp('file').join('testdata')
    file.write(testdata)
    return Path(str(file))


@pytest.mark.anyio
async def test_read(testdatafile, testdata):
    async with await aopen(testdatafile, 'rb') as f:
        data = await f.read()

    assert f.closed
    assert data == testdata


@pytest.mark.anyio
async def test_write(testdatafile, testdata):
    async with await aopen(testdatafile, 'ab') as f:
        await f.write(b'f' * 1000)

    assert testdatafile.stat().st_size == len(testdata) + 1000


@pytest.mark.anyio
async def test_async_iteration(tmpdir):
    lines = ['blah blah\n', 'foo foo\n', 'bar bar']
    testpath = tmpdir.join('testfile')
    testpath.write_text(''.join(lines), 'ascii')
    async with await aopen(str(testpath)) as f:
        lines_i = iter(lines)
        async for line in f:
            assert line == next(lines_i)



import os
import signal
import sys

import pytest

from anyio import receive_signals, run_in_thread


@pytest.mark.skipif(sys.platform == 'win32', reason='Signal delivery cannot be tested on Windows')
@pytest.mark.anyio
async def test_receive_signals():
    async with receive_signals(signal.SIGUSR1, signal.SIGUSR2) as sigiter:
        await run_in_thread(os.kill, os.getpid(), signal.SIGUSR1)
        await run_in_thread(os.kill, os.getpid(), signal.SIGUSR2)
        assert await sigiter.__anext__() == signal.SIGUSR1
        assert await sigiter.__anext__() == signal.SIGUSR2



import sys
from contextlib import contextmanager
from functools import partial
from typing import Callable, Set, List, Optional, Awaitable  # noqa: F401

import curio.io
import curio.socket
import curio.ssl
import curio.traps
from async_generator import async_generator, asynccontextmanager, yield_

from .._networking import BaseSocket
from .. import abc, T_Retval, claim_current_thread, _local
from ..exceptions import ExceptionGroup, CancelledError


def run(func: Callable[..., T_Retval], *args) -> T_Retval:
    kernel = None
    try:
        with curio.Kernel() as kernel:
            return kernel.run(func, *args)
    except BaseException:
        if kernel:
            kernel.run(shutdown=True)

        raise


@contextmanager
def translate_exceptions():
    try:
        yield
    except (curio.CancelledError, curio.TaskCancelled) as exc:
        raise CancelledError().with_traceback(exc.__traceback__) from None


#
# Timeouts and cancellation
#

class CurioCancelScope(abc.CancelScope):
    __slots__ = 'children', '_tasks', '_cancel_called'

    def __init__(self) -> None:
        self.children = set()  # type: Set[CurioCancelScope]
        self._tasks = set()  # type: Set[curio.Task]
        self._cancel_called = False

    def add_task(self, task: curio.Task) -> None:
        self._tasks.add(task)

    def remove_task(self, task: curio.Task) -> None:
        self._tasks.remove(task)

    async def cancel(self):
        if not self._cancel_called:
            self._cancel_called = True

            for task in self._tasks:
                if task.coro.cr_await is not None:
                    await task.cancel(blocking=False)

            for scope in self.children:
                await scope.cancel()


def get_cancel_scope(task: curio.Task) -> Optional[CurioCancelScope]:
    try:
        return _local.cancel_scopes_by_task.get(task)
    except AttributeError:
        return None


def set_cancel_scope(task: curio.Task, scope: Optional[CurioCancelScope]) -> None:
    try:
        cancel_scopes = _local.cancel_scopes_by_task
    except AttributeError:
        cancel_scopes = _local.cancel_scopes_by_task = {}

    if scope is None:
        del cancel_scopes[task]
    else:
        cancel_scopes[task] = scope


async def check_cancelled():
    task = await curio.current_task()
    cancel_scope = get_cancel_scope(task)
    if cancel_scope is not None and cancel_scope._cancel_called:
        raise CancelledError


async def sleep(seconds: int):
    await check_cancelled()
    await curio.sleep(seconds)


@asynccontextmanager
@async_generator
async def open_cancel_scope():
    await check_cancelled()
    task = await curio.current_task()
    scope = CurioCancelScope()
    scope.add_task(task)
    parent_scope = get_cancel_scope(task)
    if parent_scope is not None:
        parent_scope.children.add(scope)

    set_cancel_scope(task, scope)
    try:
        await yield_(scope)
    finally:
        if parent_scope is not None:
            parent_scope.children.remove(scope)
            set_cancel_scope(task, scope)
        else:
            set_cancel_scope(task, None)


@asynccontextmanager
@async_generator
async def fail_after(delay: float):
    async with open_cancel_scope() as cancel_scope:
        async with curio.ignore_after(delay) as s:
            await yield_()

        if s.expired:
            await cancel_scope.cancel()
            raise TimeoutError


@asynccontextmanager
@async_generator
async def move_on_after(delay: float):
    async with curio.ignore_after(delay):
        await yield_()


#
# Task groups
#

class CurioTaskGroup:
    __slots__ = 'cancel_scope', '_active', '_tasks', '_host_task', '_exceptions'

    def __init__(self, cancel_scope: 'CurioCancelScope', host_task: curio.Task) -> None:
        self.cancel_scope = cancel_scope
        self._host_task = host_task
        self._active = True
        self._exceptions = []  # type: List[BaseException]
        self._tasks = set()  # type: Set[curio.Task]

    async def spawn(self, func: Callable, *args, name=None) -> None:
        if not self._active:
            raise RuntimeError('This task group is not active; no new tasks can be spawned.')

        task = await curio.spawn(func, *args, report_crash=False)
        task._taskgroup = self
        self._tasks.add(task)
        if name is not None:
            task.name = name

        # Make the spawned task inherit the current cancel scope
        current_task = await curio.current_task()
        cancel_scope = get_cancel_scope(current_task)
        cancel_scope.add_task(task)
        set_cancel_scope(task, cancel_scope)

    async def _task_done(self, task: curio.Task) -> None:
        self._tasks.remove(task)

    def _task_discard(self, task: curio.Task) -> None:
        # Remove the task from its cancel scope
        cancel_scope = get_cancel_scope(task)  # type: CurioCancelScope
        if cancel_scope is not None:
            cancel_scope.remove_task(task)

        self._tasks.discard(task)
        if task.terminated and task.exception is not None:
            if not isinstance(task.exception, (CancelledError, curio.TaskCancelled,
                                               curio.CancelledError)):
                self._exceptions.append(task.exception)


@asynccontextmanager
@async_generator
async def create_task_group():
    async with open_cancel_scope() as cancel_scope:
        current_task = await curio.current_task()
        group = CurioTaskGroup(cancel_scope, current_task)
        try:
            with translate_exceptions():
                await yield_(group)
        except CancelledError:
            await cancel_scope.cancel()
        except BaseException as exc:
            group._exceptions.append(exc)
            await cancel_scope.cancel()

        while group._tasks:
            for task in set(group._tasks):
                await task.wait()

        group._active = False
        if len(group._exceptions) > 1:
            raise ExceptionGroup(group._exceptions)
        elif group._exceptions:
            raise group._exceptions[0]


#
# Threads
#

async def run_in_thread(func: Callable[..., T_Retval], *args) -> T_Retval:
    def wrapper():
        asynclib = sys.modules[__name__]
        with claim_current_thread(asynclib):
            return func(*args)

    await check_cancelled()
    thread = await curio.spawn_thread(wrapper)
    try:
        return await thread.join()
    except curio.TaskError as exc:
        raise exc.__cause__ from None


def run_async_from_thread(func: Callable[..., T_Retval], *args) -> T_Retval:
    return curio.AWAIT(func(*args))


#
# Async file I/O
#

async def aopen(*args, **kwargs):
    fp = await run_in_thread(partial(open, *args, **kwargs))
    return curio.file.AsyncFile(fp)


#
# Networking
#

class Socket(BaseSocket):
    def _wait_readable(self) -> None:
        return wait_socket_readable(self._raw_socket)

    def _wait_writable(self) -> None:
        return wait_socket_writable(self._raw_socket)

    def _check_cancelled(self) -> Awaitable[None]:
        return check_cancelled()

    def _run_in_thread(self, func: Callable, *args):
        return run_in_thread(func, *args)


def wait_socket_readable(sock):
    return curio.traps._read_wait(sock)


def wait_socket_writable(sock):
    return curio.traps._write_wait(sock)


#
# Signal handling
#

@asynccontextmanager
@async_generator
async def receive_signals(*signals: int):
    async with curio.SignalQueue(*signals) as queue:
        await yield_(queue)


#
# Synchronization
#

class Lock(curio.Lock):
    async def __aenter__(self):
        await check_cancelled()
        return await super().__aenter__()


class Condition(curio.Condition):
    async def __aenter__(self):
        await check_cancelled()
        return await super().__aenter__()

    async def wait(self):
        await check_cancelled()
        return await super().wait()


class Event(curio.Event):
    async def wait(self):
        await check_cancelled()
        return await super().wait()


class Semaphore(curio.Semaphore):
    async def __aenter__(self):
        await check_cancelled()
        return await super().__aenter__()


class Queue(curio.Queue):
    async def get(self):
        await check_cancelled()
        return await super().get()

    async def put(self, item):
        await check_cancelled()
        return await super().put(item)


abc.TaskGroup.register(CurioTaskGroup)
abc.Lock.register(Lock)
abc.Condition.register(Condition)
abc.Event.register(Event)
abc.Semaphore.register(Semaphore)
abc.Queue.register(Queue)



import asyncio
import inspect
import os
import socket
import sys
from contextlib import suppress
from functools import partial
from threading import Thread
from typing import Callable, Set, Optional, List, Union  # noqa: F401

from async_generator import async_generator, yield_, asynccontextmanager

from .._networking import BaseSocket
from .. import abc, claim_current_thread, _local, T_Retval
from ..exceptions import ExceptionGroup, CancelledError

try:
    from asyncio import run as native_run, create_task, get_running_loop, current_task
except ImportError:
    # Snatched from the standard library
    def native_run(main, *, debug=False):
        """Run a coroutine.

        This function runs the passed coroutine, taking care of
        managing the asyncio event loop and finalizing asynchronous
        generators.

        This function cannot be called when another asyncio event loop is
        running in the same thread.

        If debug is True, the event loop will be run in debug mode.

        This function always creates a new event loop and closes it at the end.
        It should be used as a main entry point for asyncio programs, and should
        ideally only be called once.

        Example:

            async def main():
                await asyncio.sleep(1)
                print('hello')

            asyncio.run(main())
        """
        from asyncio import events, coroutines

        if events._get_running_loop() is not None:
            raise RuntimeError(
                "asyncio.run() cannot be called from a running event loop")

        if not coroutines.iscoroutine(main):
            raise ValueError("a coroutine was expected, got {!r}".format(main))

        loop = events.new_event_loop()
        try:
            events.set_event_loop(loop)
            loop.set_debug(debug)
            return loop.run_until_complete(main)
        finally:
            try:
                _cancel_all_tasks(loop)
                loop.run_until_complete(loop.shutdown_asyncgens())
            finally:
                events.set_event_loop(None)
                loop.close()

    def _cancel_all_tasks(loop):
        from asyncio import Task, gather

        to_cancel = Task.all_tasks(loop)
        if not to_cancel:
            return

        for task in to_cancel:
            task.cancel()

        loop.run_until_complete(
            gather(*to_cancel, loop=loop, return_exceptions=True))

        for task in to_cancel:
            if task.cancelled():
                continue
            if task.exception() is not None:
                loop.call_exception_handler({
                    'message': 'unhandled exception during asyncio.run() shutdown',
                    'exception': task.exception(),
                    'task': task,
                })

    def create_task(coro) -> asyncio.Task:
        return get_running_loop().create_task(coro)

    def get_running_loop():
        loop = asyncio._get_running_loop()
        if loop is not None:
            return loop
        else:
            raise RuntimeError('no running event loop')

    current_task = asyncio.Task.current_task

_create_task_supports_name = 'name' in inspect.signature(create_task).parameters


def run(func: Callable[..., T_Retval], *args, debug: bool = False,
        policy: Optional[asyncio.AbstractEventLoopPolicy] = None) -> T_Retval:
    if policy is not None:
        asyncio.set_event_loop_policy(policy)

    return native_run(func(*args), debug=debug)


#
# Timeouts and cancellation
#

class AsyncIOCancelScope(abc.CancelScope):
    __slots__ = 'children', '_tasks', '_cancel_called'

    def __init__(self) -> None:
        self.children = set()  # type: Set[AsyncIOCancelScope]
        self._tasks = set()  # type: Set[asyncio.Task]
        self._cancel_called = False

    def add_task(self, task: asyncio.Task) -> None:
        self._tasks.add(task)
        set_cancel_scope(task, self)
        task.add_done_callback(self._task_done)

    def _task_done(self, task: asyncio.Task) -> None:
        self._tasks.remove(task)
        set_cancel_scope(task, None)

    async def cancel(self):
        if not self._cancel_called:
            self._cancel_called = True

            # Cancel all tasks that have started (i.e. they're awaiting on something)
            for task in self._tasks:
                coro = task._coro  # dirty, but works with both the stdlib event loop and uvloop
                if coro.cr_await is not None:
                    task.cancel()

            for child in self.children:
                await child.cancel()


def get_cancel_scope(task: asyncio.Task) -> Optional[AsyncIOCancelScope]:
    try:
        return _local.cancel_scopes_by_task.get(task)
    except AttributeError:
        return None


def set_cancel_scope(task: asyncio.Task, scope: Optional[AsyncIOCancelScope]):
    try:
        cancel_scopes = _local.cancel_scopes_by_task
    except AttributeError:
        cancel_scopes = _local.cancel_scopes_by_task = {}

    if scope is None:
        del cancel_scopes[task]
    else:
        cancel_scopes[task] = scope


def check_cancelled():
    task = current_task()
    cancel_scope = get_cancel_scope(task)
    if cancel_scope is not None and cancel_scope._cancel_called:
        raise CancelledError


async def sleep(delay: float) -> None:
    check_cancelled()
    await asyncio.sleep(delay)


@asynccontextmanager
@async_generator
async def open_cancel_scope():
    task = current_task()
    scope = AsyncIOCancelScope()
    scope.add_task(task)
    parent_scope = get_cancel_scope(task)
    if parent_scope is not None:
        parent_scope.children.add(scope)

    try:
        await yield_(scope)
    finally:
        if parent_scope is not None:
            parent_scope.children.remove(scope)
            set_cancel_scope(task, parent_scope)
        else:
            set_cancel_scope(task, None)


@asynccontextmanager
@async_generator
async def fail_after(delay: float):
    async def timeout():
        nonlocal timeout_expired
        await sleep(delay)
        timeout_expired = True
        await scope.cancel()

    timeout_expired = False
    async with open_cancel_scope() as scope:
        timeout_task = get_running_loop().create_task(timeout())
        try:
            await yield_(scope)
        except asyncio.CancelledError as exc:
            if timeout_expired:
                await scope.cancel()
                raise TimeoutError().with_traceback(exc.__traceback__) from None
            else:
                raise
        finally:
            timeout_task.cancel()
            await asyncio.gather(timeout_task, return_exceptions=True)


@asynccontextmanager
@async_generator
async def move_on_after(delay: float):
    async def timeout():
        nonlocal timeout_expired
        await sleep(delay)
        timeout_expired = True
        await scope.cancel()

    timeout_expired = False
    async with open_cancel_scope() as scope:
        timeout_task = get_running_loop().create_task(timeout())
        try:
            await yield_(scope)
        except asyncio.CancelledError:
            if timeout_expired:
                await scope.cancel()
            else:
                raise
        finally:
            timeout_task.cancel()
            await asyncio.gather(timeout_task, return_exceptions=True)


#
# Task groups
#

class AsyncIOTaskGroup:
    __slots__ = 'cancel_scope', '_active', '_tasks', '_host_task', '_exceptions'

    def __init__(self, cancel_scope: 'AsyncIOCancelScope', host_task: asyncio.Task) -> None:
        self.cancel_scope = cancel_scope
        self._host_task = host_task
        self._active = True
        self._exceptions = []  # type: List[BaseException]
        self._tasks = set()  # type: Set[asyncio.Task]

    def _task_done(self, task: asyncio.Task) -> None:
        self._tasks.remove(task)
        if not task.cancelled():
            exc = task.exception()
            if exc is not None and not isinstance(exc, CancelledError):
                self._exceptions.append(exc)
                self._host_task.cancel()

    async def spawn(self, func: Callable, *args, name=None) -> None:
        if not self._active:
            raise RuntimeError('This task group is not active; no new tasks can be spawned.')

        if _create_task_supports_name:
            task = create_task(func(*args), name=name)
        else:
            task = create_task(func(*args))

        self._tasks.add(task)
        task.add_done_callback(self._task_done)

        # Make the spawned task inherit the task group's cancel scope
        set_cancel_scope(task, self.cancel_scope)
        self.cancel_scope.add_task(task)


@asynccontextmanager
@async_generator
async def create_task_group():
    async with open_cancel_scope() as cancel_scope:
        group = AsyncIOTaskGroup(cancel_scope, current_task())
        try:
            await yield_(group)
        except CancelledError:
            await cancel_scope.cancel()
        except BaseException as exc:
            group._exceptions.append(exc)
            await cancel_scope.cancel()

        while group._tasks:
            with suppress(asyncio.CancelledError):
                await asyncio.wait(group._tasks)

        group._active = False
        if len(group._exceptions) > 1:
            raise ExceptionGroup(group._exceptions)
        elif group._exceptions:
            raise group._exceptions[0]


#
# Threads
#

async def run_in_thread(func: Callable[..., T_Retval], *args) -> T_Retval:
    def thread_worker():
        try:
            with claim_current_thread(asynclib):
                _local.loop = loop
                result = func(*args)
        except BaseException as exc:
            loop.call_soon_threadsafe(queue.put_nowait, (None, exc))
        else:
            loop.call_soon_threadsafe(queue.put_nowait, (result, None))

    check_cancelled()
    asynclib = sys.modules[__name__]
    loop = get_running_loop()
    queue = asyncio.Queue(1)
    thread = Thread(target=thread_worker)
    thread.start()
    retval, exception = await queue.get()
    if exception is not None:
        raise exception
    else:
        return retval


def run_async_from_thread(func: Callable[..., T_Retval], *args) -> T_Retval:
    f = asyncio.run_coroutine_threadsafe(func(*args), _local.loop)
    return f.result()


#
# Async file I/O
#

class AsyncFile:
    def __init__(self, fp) -> None:
        self._fp = fp

    def __getattr__(self, name):
        return getattr(self._fp, name)

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()

    @async_generator
    async def __aiter__(self):
        while True:
            line = await self.readline()
            if line:
                await yield_(line)
            else:
                break

    async def read(self, size: int = -1) -> Union[bytes, str]:
        return await run_in_thread(self._fp.read, size)

    async def read1(self, size: int = -1) -> Union[bytes, str]:
        return await run_in_thread(self._fp.read1, size)

    async def readline(self) -> bytes:
        return await run_in_thread(self._fp.readline)

    async def readlines(self) -> bytes:
        return await run_in_thread(self._fp.readlines)

    async def readinto(self, b: Union[bytes, memoryview]) -> bytes:
        return await run_in_thread(self._fp.readinto, b)

    async def readinto1(self, b: Union[bytes, memoryview]) -> bytes:
        return await run_in_thread(self._fp.readinto1, b)

    async def write(self, b: bytes) -> None:
        return await run_in_thread(self._fp.write, b)

    async def writelines(self, lines: bytes) -> None:
        return await run_in_thread(self._fp.writelines, lines)

    async def truncate(self, size: Optional[int] = None) -> int:
        return await run_in_thread(self._fp.truncate, size)

    async def seek(self, offset: int, whence: Optional[int] = os.SEEK_SET) -> int:
        return await run_in_thread(self._fp.seek, offset, whence)

    async def tell(self) -> int:
        return await run_in_thread(self._fp.tell)

    async def flush(self) -> None:
        return await run_in_thread(self._fp.flush)

    async def close(self) -> None:
        return await run_in_thread(self._fp.close)


async def aopen(*args, **kwargs):
    fp = await run_in_thread(partial(open, *args, **kwargs))
    return AsyncFile(fp)


#
# Networking
#

class Socket(BaseSocket):
    __slots__ = '_loop', '_read_event', '_write_event'

    def __init__(self, raw_socket: socket.SocketType) -> None:
        self._loop = get_running_loop()
        self._read_event = asyncio.Event(loop=self._loop)
        self._write_event = asyncio.Event(loop=self._loop)
        super().__init__(raw_socket)

    async def _wait_readable(self) -> None:
        check_cancelled()
        self._loop.add_reader(self._raw_socket, self._read_event.set)
        try:
            await self._read_event.wait()
            self._read_event.clear()
        finally:
            self._loop.remove_reader(self._raw_socket)

    async def _wait_writable(self) -> None:
        check_cancelled()
        self._loop.add_writer(self._raw_socket, self._write_event.set)
        try:
            await self._write_event.wait()
            self._write_event.clear()
        finally:
            self._loop.remove_writer(self._raw_socket)

    async def _check_cancelled(self) -> None:
        check_cancelled()

    def _run_in_thread(self, func: Callable, *args):
        return run_in_thread(func, *args)


async def wait_socket_readable(sock: socket.SocketType) -> None:
    check_cancelled()
    loop = get_running_loop()
    event = asyncio.Event(loop=loop)
    loop.add_reader(sock, event.set)
    try:
        await event.wait()
    finally:
        loop.remove_reader(sock)


async def wait_socket_writable(sock: socket.SocketType) -> None:
    check_cancelled()
    loop = get_running_loop()
    event = asyncio.Event(loop=loop)
    loop.add_writer(sock.fileno(), event.set)
    try:
        await event.wait()
    finally:
        loop.remove_writer(sock)


#
# Synchronization
#

class Lock(asyncio.Lock):
    async def __aenter__(self):
        check_cancelled()
        await self.acquire()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self.release()


class Condition(asyncio.Condition):
    async def __aenter__(self):
        check_cancelled()
        return await super().__aenter__()

    async def notify(self, n=1):
        super().notify(n)

    async def notify_all(self):
        super().notify(len(self._waiters))

    def wait(self):
        check_cancelled()
        return super().wait()


class Event(asyncio.Event):
    async def set(self):
        super().set()

    def wait(self):
        check_cancelled()
        return super().wait()


class Semaphore(asyncio.Semaphore):
    def __aenter__(self):
        check_cancelled()
        return super().__aenter__()

    @property
    def value(self):
        return self._value


class Queue(asyncio.Queue):
    def get(self):
        check_cancelled()
        return super().get()

    def put(self, item):
        check_cancelled()
        return super().put(item)


abc.TaskGroup.register(AsyncIOTaskGroup)
abc.Lock.register(Lock)
abc.Condition.register(Condition)
abc.Event.register(Event)
abc.Semaphore.register(Semaphore)
abc.Queue.register(Queue)


#
# Signal handling
#

@asynccontextmanager
@async_generator
async def receive_signals(*signals: int):
    @async_generator
    async def process_signal_queue():
        while True:
            signum = await queue.get()
            await yield_(signum)

    loop = get_running_loop()
    queue = asyncio.Queue(loop=loop)
    handled_signals = set()
    try:
        for sig in set(signals):
            loop.add_signal_handler(sig, queue.put_nowait, sig)
            handled_signals.add(sig)

        await yield_(process_signal_queue())
    finally:
        for sig in handled_signals:
            loop.remove_signal_handler(sig)



import sys
from functools import wraps
from typing import Callable

import trio.hazmat
from async_generator import async_generator, yield_, asynccontextmanager

from .._networking import BaseSocket
from .. import abc, claim_current_thread, T_Retval, _local
from ..exceptions import ExceptionGroup


class DummyAwaitable:
    def __await__(self):
        if False:
            yield


dummy_awaitable = DummyAwaitable()


def wrap_as_awaitable(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        func(*args, **kwargs)
        return dummy_awaitable

    return wrapper


#
# Timeouts and cancellation
#

@asynccontextmanager
@async_generator
async def open_cancel_scope():
    with trio.open_cancel_scope() as cancel_scope:
        cancel_scope.cancel = wrap_as_awaitable(cancel_scope.cancel)
        await yield_(cancel_scope)


@asynccontextmanager
@async_generator
async def move_on_after(seconds):
    with trio.move_on_after(seconds) as s:
        await yield_(s)


@asynccontextmanager
@async_generator
async def fail_after(seconds):
    try:
        with trio.fail_after(seconds) as s:
            await yield_(s)
    except trio.TooSlowError as exc:
        raise TimeoutError().with_traceback(exc.__traceback__) from None


#
# Task groups
#

class TaskGroup:
    __slots__ = '_active', '_nursery'

    def __init__(self, nursery) -> None:
        self._active = True
        self._nursery = nursery
        nursery.cancel_scope.cancel = wrap_as_awaitable(nursery.cancel_scope.cancel)

    @property
    def cancel_scope(self):
        return self._nursery.cancel_scope

    async def spawn(self, func: Callable, *args, name=None) -> None:
        if not self._active:
            raise RuntimeError('This task group is not active; no new tasks can be spawned.')

        self._nursery.start_soon(func, *args, name=name)


@asynccontextmanager
@async_generator
async def create_task_group():
    try:
        async with trio.open_nursery() as nursery:
            tg = TaskGroup(nursery)
            await yield_(tg)
            tg._active = False
    except trio.MultiError as exc:
        raise ExceptionGroup(exc.exceptions) from None


#
# Threads
#

async def run_in_thread(func: Callable[..., T_Retval], *args) -> T_Retval:
    def wrapper():
        asynclib = sys.modules[__name__]
        with claim_current_thread(asynclib):
            _local.portal = portal
            return func(*args)

    portal = trio.BlockingTrioPortal()
    return await trio.run_sync_in_worker_thread(wrapper)


def run_async_from_thread(func: Callable[..., T_Retval], *args) -> T_Retval:
    return _local.portal.run(func, *args)


#
# Async file I/O
#

aopen = trio.open_file


#
# Networking
#

class Socket(BaseSocket):
    __slots__ = ()

    def _wait_readable(self):
        return wait_socket_readable(self._raw_socket)

    def _wait_writable(self) -> None:
        return wait_socket_writable(self._raw_socket)

    def _check_cancelled(self) -> None:
        return trio.hazmat.checkpoint_if_cancelled()

    def _run_in_thread(self, func: Callable, *args):
        return run_in_thread(func, *args)


wait_socket_readable = trio.hazmat.wait_socket_readable
wait_socket_writable = trio.hazmat.wait_socket_writable


#
# Signal handling
#

@asynccontextmanager
@async_generator
async def receive_signals(*signals: int):
    with trio.open_signal_receiver(*signals) as cm:
        await yield_(cm)


#
# Synchronization
#

class Event(trio.Event):
    async def set(self) -> None:
        super().set()


class Condition(trio.Condition):
    async def notify(self, n: int = 1) -> None:
        super().notify(n)

    async def notify_all(self) -> None:
        super().notify_all()


run = trio.run
sleep = trio.sleep

Lock = trio.Lock
Semaphore = trio.Semaphore
Queue = trio.Queue

abc.TaskGroup.register(TaskGroup)
abc.Lock.register(Lock)
abc.Condition.register(Condition)
abc.Event.register(Event)
abc.Semaphore.register(Semaphore)
abc.Queue.register(Queue)



from typing import Sequence


class ExceptionGroup(Exception):
    """Raised when multiple exceptions have been raised in a task group."""

    def __init__(self, exceptions: Sequence[BaseException]) -> None:
        super().__init__(exceptions)

        self.msg = ''
        for i, exc in enumerate(exceptions, 1):
            if i > 1:
                self.msg += '\n\n'

            self.msg += 'Details of embedded exception {}:\n\n  {}: {}'.format(
                i, exc.__class__.__name__, exc)

    @property
    def exceptions(self) -> Sequence[BaseException]:
        return self.args[0]

    def __str__(self):
        return ', '.join('{}{}'.format(exc.__class__.__name__, exc.args) for exc in self.args[0])

    def __repr__(self) -> str:
        return '<{}: {}>'.format(self.__class__.__name__,
                                 ', '.join(repr(exc) for exc in self.args[0]))


class CancelledError(Exception):
    """Raised when the enclosing cancel scope has been cancelled."""


class IncompleteRead(Exception):
    """"
    Raised during ``read_exactly()`` if the connection is closed before the requested amount of
    bytes has been read.

    :ivar bytes data: bytes read before the stream was closed
    """

    def __init__(self, data: bytes) -> None:
        super().__init__('The stream was closed before the read operation could be completed')
        self.data = data


class DelimiterNotFound(Exception):
    """
    Raised during ``read_until()`` if the maximum number of bytes has been read without the
    delimiter being found.

    :ivar bytes data: bytes read before giving up
    """

    def __init__(self, data: bytes) -> None:
        super().__init__(
            'The delimiter was not found among the first {} bytes read'.format(len(data)))
        self.data = data



import pytest

from anyio import (
    run_async_from_thread, run_in_thread, is_in_event_loop_thread, create_task_group)


@pytest.mark.anyio
async def test_run_async_from_thread():
    async def add(a, b):
        assert is_in_event_loop_thread()
        return a + b

    def worker(a, b):
        assert not is_in_event_loop_thread()
        return run_async_from_thread(add, a, b)

    result = await run_in_thread(worker, 1, 2)
    assert result == 3


@pytest.mark.anyio
async def test_run_in_thread_cancelled():
    def thread_worker():
        nonlocal state
        state = 2

    async def worker():
        nonlocal state
        state = 1
        await run_in_thread(thread_worker)
        state = 3

    state = 0
    async with create_task_group() as tg:
        await tg.spawn(worker)
        await tg.cancel_scope.cancel()

    assert state == 1


@pytest.mark.anyio
async def test_run_in_thread_exception():
    def thread_worker():
        raise ValueError('foo')

    with pytest.raises(ValueError) as exc:
        await run_in_thread(thread_worker)

    exc.match('^foo$')



import pytest

from anyio import (
    create_lock, create_task_group, sleep, create_queue, create_event, create_semaphore,
    create_condition, open_cancel_scope)


class TestLock:
    @pytest.mark.anyio
    async def test_lock(self):
        async def task():
            assert lock.locked()
            async with lock:
                results.append('2')

        results = []
        lock = create_lock()
        async with create_task_group() as tg:
            async with lock:
                await tg.spawn(task)
                await sleep(0.1)
                results.append('1')

        assert not lock.locked()
        assert results == ['1', '2']

    @pytest.mark.anyio
    async def test_lock_cancel(self):
        async def task():
            nonlocal task_started, got_lock
            task_started = True
            async with lock:
                got_lock = True

        task_started = got_lock = False
        lock = create_lock()
        async with create_task_group() as tg:
            async with lock:
                await tg.spawn(task)
                await tg.cancel_scope.cancel()

        assert task_started
        assert not got_lock


class TestEvent:
    @pytest.mark.anyio
    async def test_event(self):
        async def setter():
            assert not event.is_set()
            await event.set()

        event = create_event()
        async with create_task_group() as tg:
            await tg.spawn(setter)
            await event.wait()
            event.clear()

        assert not event.is_set()

    @pytest.mark.anyio
    async def test_event_cancel(self):
        async def task():
            nonlocal task_started, event_set
            task_started = True
            await event.wait()
            event_set = True

        task_started = event_set = False
        event = create_event()
        async with create_task_group() as tg:
            await tg.spawn(task)
            await tg.cancel_scope.cancel()
            await event.set()

        assert task_started
        assert not event_set


class TestCondition:
    @pytest.mark.anyio
    async def test_condition(self):
        async def notifier():
            async with condition:
                await condition.notify_all()

        condition = create_condition()
        async with create_task_group() as tg:
            async with condition:
                assert condition.locked()
                await tg.spawn(notifier)
                await condition.wait()

    @pytest.mark.anyio
    async def test_wait_cancel(self):
        async def task():
            nonlocal task_started, notified
            task_started = True
            async with condition:
                await event.set()
                event.clear()
                await event.wait()
                await condition.wait()
                notified = True

        task_started = notified = False
        event = create_event()
        condition = create_condition()
        async with create_task_group() as tg:
            await tg.spawn(task)
            await event.wait()
            await tg.cancel_scope.cancel()
            await event.set()

        assert task_started
        assert not notified


class TestSemaphore:
    @pytest.mark.anyio
    async def test_semaphore(self):
        async def acquire():
            async with semaphore:
                assert semaphore.value in (0, 1)

        semaphore = create_semaphore(2)
        async with create_task_group() as tg:
            await tg.spawn(acquire, name='task 1')
            await tg.spawn(acquire, name='task 2')

        assert semaphore.value == 2

    @pytest.mark.anyio
    async def test_acquire_cancel(self):
        async def task():
            nonlocal local_scope, acquired
            async with open_cancel_scope() as local_scope, semaphore:
                acquired = True

        local_scope = acquired = None
        semaphore = create_semaphore(1)
        async with create_task_group() as tg:
            async with semaphore:
                await tg.spawn(task)
                while local_scope is None:
                    await sleep(0)

                await local_scope.cancel()

        assert not acquired


class TestQueue:
    @pytest.mark.anyio
    async def test_queue(self):
        queue = create_queue(1)
        assert queue.empty()
        await queue.put('1')
        assert queue.full()
        assert queue.qsize() == 1
        assert await queue.get() == '1'
        assert queue.empty()

    @pytest.mark.anyio
    async def test_get_cancel(self):
        async def task():
            nonlocal local_scope
            async with open_cancel_scope() as local_scope:
                await queue.get()

        local_scope = None
        queue = create_queue(1)
        async with create_task_group() as tg:
            await tg.spawn(task)
            while local_scope is None:
                await sleep(0)

            await local_scope.cancel()
            await queue.put(None)

        assert queue.full()

    @pytest.mark.anyio
    async def test_put_cancel(self):
        async def task():
            nonlocal local_scope
            async with open_cancel_scope() as local_scope:
                await queue.put(None)

        local_scope = None
        queue = create_queue(1)
        await queue.put(None)
        async with create_task_group() as tg:
            await tg.spawn(task)
            while local_scope is None:
                await sleep(0)

            await local_scope.cancel()
            await queue.get()

        assert queue.empty()



import ssl
import sys
from pathlib import Path

import pytest

from anyio import (
    create_task_group, connect_tcp, create_udp_socket, connect_unix, create_unix_server,
    create_tcp_server)
from anyio.exceptions import IncompleteRead, DelimiterNotFound


@pytest.mark.anyio
async def test_connect_tcp():
    async def server():
        async with await stream_server.accept() as stream:
            command = await stream.receive_some(100)
            await stream.send_all(command[::-1])

    async with create_task_group() as tg:
        async with await create_tcp_server(interface='localhost') as stream_server:
            await tg.spawn(server)
            async with await connect_tcp('localhost', stream_server.port) as client:
                await client.send_all(b'blah')
                response = await client.receive_some(100)

    assert response == b'halb'


@pytest.mark.anyio
async def test_connect_tcp_tls():
    async def server():
        async with await stream_server.accept() as stream:
            command = await stream.receive_some(100)
            await stream.send_all(command[::-1])

    server_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
    server_context.load_cert_chain(certfile=str(Path(__file__).with_name('cert.pem')),
                                   keyfile=str(Path(__file__).with_name('key.pem')))
    client_context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
    client_context.load_verify_locations(cafile=str(Path(__file__).with_name('cert.pem')))
    async with create_task_group() as tg:
        async with await create_tcp_server(
                interface='localhost', ssl_context=server_context) as stream_server:
            await tg.spawn(server)
            async with await connect_tcp('localhost', stream_server.port,
                                         tls=client_context) as client:
                await client.send_all(b'blah')
                response = await client.receive_some(100)

    assert response == b'halb'


@pytest.mark.skipif(sys.platform == 'win32', reason='UNIX sockets are not available on Windows')
@pytest.mark.parametrize('as_path', [False])
@pytest.mark.anyio
async def test_connect_unix(tmpdir, as_path):
    async def server():
        async with await stream_server.accept() as stream:
            command = await stream.receive_some(100)
            await stream.send_all(command[::-1])

    async with create_task_group() as tg:
        path = str(tmpdir.join('socket'))
        if as_path:
            path = Path(path)

        async with await create_unix_server(path) as stream_server:
            await tg.spawn(server)
            async with await connect_unix(path) as client:
                await client.send_all(b'blah')
                response = await client.receive_some(100)

    assert response == b'halb'


@pytest.mark.parametrize('method_name, params', [
    ('receive_until', [b'\n', 100]),
    ('receive_exactly', [5])
], ids=['read_until', 'read_exactly'])
@pytest.mark.anyio
async def test_read_partial(method_name, params):
    async def server():
        async with await stream_server.accept() as stream:
            method = getattr(stream, method_name)
            line1 = await method(*params)
            line2 = await method(*params)
            await stream.send_all(line1.strip() + line2.strip())

    async with create_task_group() as tg:
        async with await create_tcp_server(interface='localhost') as stream_server:
            await tg.spawn(server)
            async with await connect_tcp('localhost', stream_server.port) as client:
                await client.send_all(b'bla')
                await client.send_all(b'h\nb')
                await client.send_all(b'leh\n')
                response = await client.receive_some(100)

    assert response == b'blahbleh'


@pytest.mark.parametrize('method_name, params', [
    ('receive_until', [b'\n', 100]),
    ('receive_exactly', [5])
], ids=['read_until', 'read_exactly'])
@pytest.mark.anyio
async def test_incomplete_read(method_name, params):
    async def server():
        async with await stream_server.accept() as stream:
            await stream.send_all(b'bla')

    async with create_task_group() as tg:
        async with await create_tcp_server(interface='localhost') as stream_server:
            await tg.spawn(server)
            async with await connect_tcp('localhost', stream_server.port) as client:
                method = getattr(client, method_name)
                with pytest.raises(IncompleteRead) as exc:
                    await method(*params)

                assert exc.value.data == b'bla'


@pytest.mark.anyio
async def test_delimiter_not_found():
    async def server():
        async with await stream_server.accept() as stream:
            await stream.send_all(b'blah\n')

    async with create_task_group() as tg:
        async with await create_tcp_server(interface='localhost') as stream_server:
            await tg.spawn(server)
            async with await connect_tcp('localhost', stream_server.port) as client:
                with pytest.raises(DelimiterNotFound) as exc:
                    await client.receive_until(b'\n', 3)

                assert exc.value.data == b'bla'


@pytest.mark.anyio
async def test_udp():
    async with await create_udp_socket(port=5000, interface='localhost',
                                       target_port=5000, target_host='localhost') as socket:
        await socket.send(b'blah')
        request, addr = await socket.receive(100)
        assert request == b'blah'
        assert addr == ('127.0.0.1', 5000)

        await socket.send(b'halb')
        response, addr = await socket.receive(100)
        assert response == b'halb'
        assert addr == ('127.0.0.1', 5000)


@pytest.mark.anyio
async def test_udp_noconnect():
    async with await create_udp_socket(interface='localhost') as socket:
        await socket.send(b'blah', 'localhost', socket.port)
        request, addr = await socket.receive(100)
        assert request == b'blah'
        assert addr == ('127.0.0.1', socket.port)

        await socket.send(b'halb', 'localhost', socket.port)
        response, addr = await socket.receive(100)
        assert response == b'halb'
        assert addr == ('127.0.0.1', socket.port)



import os
import socket
import ssl
import sys
import threading
import typing  # noqa: F401
from contextlib import contextmanager
from importlib import import_module
from inspect import ismodule
from pathlib import Path
from ssl import SSLContext
from typing import TypeVar, Callable, Union, Optional, Awaitable, Coroutine, Any, Dict

from .abc import (  # noqa: F401
    IPAddressType, BufferType, CancelScope, DatagramSocket, Lock, Condition, Event, Semaphore,
    Queue, TaskGroup, Stream, SocketStreamServer, SocketStream, AsyncFile)
from . import _networking

T_Retval = TypeVar('T_Retval', covariant=True)
_local = threading.local()


class NullAsyncContext:
    async def __aenter__(self) -> None:
        return None

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        pass


@contextmanager
def claim_current_thread(asynclib) -> None:
    assert ismodule(asynclib)
    _local.asynclib = asynclib
    try:
        yield
    finally:
        reset_detected_asynclib()


def reset_detected_asynclib():
    _local.__dict__.clear()


def detect_running_asynclib() -> Optional[str]:
    if 'trio' in sys.modules:
        from trio.hazmat import current_trio_token
        try:
            current_trio_token()
        except RuntimeError:
            pass
        else:
            return 'trio'

    if 'curio' in sys.modules:
        from curio.meta import curio_running
        if curio_running():
            return 'curio'

    if 'asyncio' in sys.modules:
        from ._backends.asyncio import get_running_loop
        try:
            get_running_loop()
        except RuntimeError:
            pass
        else:
            return 'asyncio'

    return None


def _get_asynclib():
    try:
        return _local.asynclib
    except AttributeError:
        asynclib_name = detect_running_asynclib()
        if asynclib_name is None:
            raise LookupError('Cannot find any running async event loop')

        _local.asynclib = import_module('{}._backends.{}'.format(__name__, asynclib_name))
        return _local.asynclib


def run(func: Callable[..., Coroutine[Any, Any, T_Retval]], *args,
        backend: str = 'asyncio', backend_options: Optional[Dict[str, Any]] = None) -> T_Retval:
    """
    Run the given coroutine function in an asynchronous event loop.

    The current thread must not be already running an event loop.

    :param func: a coroutine function
    :param args: positional arguments to ``func``
    :param backend: name of the asynchronous event loop implementation – one of ``asyncio``,
        ``curio`` and ``trio``
    :param backend_options: keyword arguments to call the backend ``run()`` implementation with
    :return: the return value of the coroutine function
    :raises RuntimeError: if an asynchronous event loop is already running in this thread
    :raises LookupError: if the named backend is not found

    """
    asynclib_name = detect_running_asynclib()
    if asynclib_name:
        raise RuntimeError('Already running {} in this thread'.format(asynclib_name))

    try:
        asynclib = import_module('{}._backends.{}'.format(__name__, backend))
    except ImportError as exc:
        raise LookupError('No such backend: {}'.format(backend)) from exc

    backend_options = backend_options or {}
    with claim_current_thread(asynclib):
        return asynclib.run(func, *args, **backend_options)


def is_in_event_loop_thread() -> bool:
    """
    Determine whether the current thread is running a recognized asynchronous event loop.

    :return: ``True`` if running in the event loop, thread, ``False`` if not

    """
    return detect_running_asynclib() is not None


#
# Timeouts and cancellation
#

def sleep(delay: float) -> Awaitable[None]:
    """
    Pause the current task for the specified duration.

    :param delay: the duration, in seconds

    """
    return _get_asynclib().sleep(delay)


def open_cancel_scope() -> 'typing.AsyncContextManager[CancelScope]':
    """
    Open a cancel scope.

    :return: an asynchronous context manager that yields a cancel scope.

    """
    return _get_asynclib().open_cancel_scope()


def fail_after(delay: Optional[float]) -> 'typing.AsyncContextManager[None]':
    """
    Create a context manager which raises an exception if does not finish in time.

    :param delay: maximum allowed time (in seconds) before raising the exception, or ``None`` to
        disable the timeout
    :return: an asynchronous context manager
    :raises TimeoutError: if the block does not complete within the allotted time

    """
    if delay is None:
        return NullAsyncContext()
    else:
        return _get_asynclib().fail_after(delay)


def move_on_after(delay: Optional[float]) -> 'typing.AsyncContextManager[None]':
    """
    Create a context manager which is exited if it does not complete within the given time.

    :param delay: maximum allowed time (in seconds) before exiting the context block, or ``None``
        to disable the timeout
    :return: an asynchronous context manager

    """
    if delay is None:
        return NullAsyncContext()
    else:
        return _get_asynclib().move_on_after(delay)


#
# Task groups
#

def create_task_group() -> 'typing.AsyncContextManager[TaskGroup]':
    """
    Create a task group.

    :return: an asynchronous context manager that yields a task group

    """
    return _get_asynclib().create_task_group()


#
# Threads
#

def run_in_thread(func: Callable[..., T_Retval], *args) -> Awaitable[T_Retval]:
    """
    Start a thread that calls the given function with the given arguments.

    :param func: a callable
    :param args: positional arguments for the callable
    :return: an awaitable that yields the return value of the function.

    """
    assert is_in_event_loop_thread()
    return _get_asynclib().run_in_thread(func, *args)


def run_async_from_thread(func: Callable[..., Coroutine[Any, Any, T_Retval]], *args) -> T_Retval:
    """
    Call a coroutine function from a worker thread.

    :param func: a coroutine function
    :param args: positional arguments for the callable
    :return: the return value of the coroutine function

    """
    assert not is_in_event_loop_thread()
    return _get_asynclib().run_async_from_thread(func, *args)


#
# Async file I/O
#

def aopen(file: Union[str, Path, int], mode: str = 'r', buffering: int = -1,
          encoding: Optional[str] = None, errors: Optional[str] = None,
          newline: Optional[str] = None, closefd: bool = True,
          opener: Optional[Callable] = None) -> Coroutine[Any, Any, AsyncFile]:
    """
    Open a file asynchronously.

    The arguments are exactly the same as for the builtin :func:`open`.

    :return: an asynchronous file object

    """
    if isinstance(file, Path):
        file = str(file)

    return _get_asynclib().aopen(file, mode, buffering, encoding, errors, newline, closefd, opener)


#
# Networking
#

def wait_socket_readable(sock: Union[socket.SocketType, ssl.SSLSocket]) -> Awaitable[None]:
    """
    Wait until the given socket has data to be read.

    :param sock: a socket object

    """
    return _get_asynclib().wait_socket_readable(sock)


def wait_socket_writable(sock: Union[socket.SocketType, ssl.SSLSocket]) -> Awaitable[None]:
    """
    Wait until the given socket can be written to.

    :param sock: a socket object

    """
    return _get_asynclib().wait_socket_writable(sock)


async def connect_tcp(
    address: IPAddressType, port: int, *, tls: Union[bool, SSLContext] = False,
    bind_host: Optional[IPAddressType] = None, bind_port: Optional[int] = None
) -> SocketStream:
    """
    Connect to a host using the TCP protocol.

    :param address: the IP address or host name to connect to
    :param port: port on the target host to connect to
    :param tls: ``False`` to not do a TLS handshake, ``True`` to do the TLS handshake using a
        default context, or an SSL context object to use for the TLS handshake
    :param bind_host: the interface address or name to bind the socket to before connecting
    :param bind_port: the port to bind the socket to before connecting
    :return: an asynchronous context manager that yields a socket stream

    """
    if bind_host:
        bind_host = str(bind_host)

    raw_socket = socket.socket()
    sock = _get_asynclib().Socket(raw_socket)
    try:
        if bind_host is not None and bind_port is not None:
            await sock.bind((bind_host, bind_port))

        await sock.connect((address, port))
        stream = _networking.SocketStream(sock, server_hostname=address)

        if isinstance(tls, SSLContext):
            await stream.start_tls(tls)
        elif tls:
            await stream.start_tls()

        return stream
    except BaseException:
        sock.close()
        raise


async def connect_unix(path: Union[str, Path]) -> SocketStream:
    """
    Connect to the given UNIX socket.

    Not available on Windows.

    :param path: path to the socket
    :return: an asynchronous context manager that yields a socket stream

    """
    raw_socket = socket.socket(socket.AF_UNIX)
    sock = _get_asynclib().Socket(raw_socket)
    try:
        await sock.connect(path)
        return _networking.SocketStream(sock)
    except BaseException:
        sock.close()
        raise


async def create_tcp_server(
    port: int = 0, interface: Optional[IPAddressType] = None,
    ssl_context: Optional[SSLContext] = None
) -> SocketStreamServer:
    """
    Start a TCP socket server.

    :param port: port number to listen on
    :param interface: interface to listen on (if omitted, listen on any interface)
    :param ssl_context: an SSL context object for TLS negotiation
    :return: an asynchronous context manager that yields a server object

    """
    if interface:
        interface = str(interface)

    raw_socket = socket.socket()
    sock = _get_asynclib().Socket(raw_socket)
    try:
        await sock.bind((interface or '', port))
        sock.listen()
        return _networking.SocketStreamServer(sock, ssl_context)
    except BaseException:
        sock.close()
        raise


async def create_unix_server(
        path: Union[str, Path], *, mode: Optional[int] = None) -> SocketStreamServer:
    """
    Start a UNIX socket server.

    Not available on Windows.

    :param path: path of the socket
    :param mode: permissions to set on the socket
    :return: an asynchronous context manager that yields a server object

    """
    raw_socket = socket.socket(socket.AF_UNIX)
    sock = _get_asynclib().Socket(raw_socket)
    try:
        await sock.bind(path)

        if mode is not None:
            os.chmod(path, mode)

        sock.listen()
        return _networking.SocketStreamServer(sock, None)
    except BaseException:
        sock.close()
        raise


async def create_udp_socket(
    *, interface: Optional[IPAddressType] = None, port: Optional[int] = None,
    target_host: Optional[IPAddressType] = None, target_port: Optional[int] = None
) -> DatagramSocket:
    """
    Create a UDP socket.

    If ``port`` has been given, the socket will be bound to this port on the local machine,
    making this socket suitable for providing UDP based services.

    :param interface: interface to bind to
    :param port: port to bind to
    :param target_host: remote host to set as the default target
    :param target_port: port on the remote host to set as the default target
    :return: a UDP socket

    """
    if interface:
        interface = str(interface)
    if target_host:
        target_host = str(target_host)

    raw_socket = socket.socket(type=socket.SOCK_DGRAM)
    sock = _get_asynclib().Socket(raw_socket)
    try:
        if interface is not None or port is not None:
            await sock.bind((interface or '', port or 0))

        if target_host is not None and target_port is not None:
            await sock.connect((target_host, target_port))

        return _networking.DatagramSocket(sock)
    except BaseException:
        sock.close()
        raise


#
# Synchronization
#

def create_lock() -> Lock:
    """
    Create an asynchronous lock.

    :return: a lock object

    """
    return _get_asynclib().Lock()


def create_condition() -> Condition:
    """
    Create an asynchronous condition.

    :return: a condition object

    """
    return _get_asynclib().Condition()


def create_event() -> Event:
    """
    Create an asynchronous event object.

    :return: an event object

    """
    return _get_asynclib().Event()


def create_semaphore(value: int) -> Semaphore:
    """
    Create an asynchronous semaphore.

    :param value: the semaphore's initial value
    :return: a semaphore object

    """
    return _get_asynclib().Semaphore(value)


def create_queue(capacity: int) -> Queue:
    """
    Create an asynchronous queue.

    :param capacity: maximum number of items the queue will be able to store
    :return: a queue object

    """
    return _get_asynclib().Queue(capacity)


#
# Signal handling
#

def receive_signals(*signals: int) -> 'typing.ContextManager[typing.AsyncIterator[int]]':
    """
    Start receiving operating system signals.

    :param signals: signals to receive (e.g. ``signal.SIGINT``)
    :return: an asynchronous context manager for an asynchronous iterator which yields signal
        numbers

    .. warning:: Windows does not support signals natively so it is best to avoid relying on this
        in cross-platform applications.

    """
    return _get_asynclib().receive_signals(*signals)



import errno
import socket
import ssl
from abc import ABCMeta, abstractmethod
from ipaddress import ip_address
from typing import Union, Tuple, Any, Optional, Callable

from anyio import abc
from anyio.abc import IPAddressType, BufferType
from anyio.exceptions import DelimiterNotFound, IncompleteRead


class BaseSocket(metaclass=ABCMeta):
    __slots__ = '_raw_socket'

    def __init__(self, raw_socket: socket.SocketType) -> None:
        self._raw_socket = raw_socket
        self._raw_socket.setblocking(False)

    def __getattr__(self, item):
        return getattr(self._raw_socket, item)

    @abstractmethod
    async def _wait_readable(self) -> None:
        pass

    @abstractmethod
    async def _wait_writable(self) -> None:
        pass

    @abstractmethod
    async def _check_cancelled(self) -> None:
        pass

    @abstractmethod
    async def _run_in_thread(self, func: Callable, *args):
        pass

    async def accept(self):
        await self._check_cancelled()
        try:
            raw_socket, address = self._raw_socket.accept()
        except BlockingIOError:
            await self._wait_readable()
            raw_socket, address = self._raw_socket.accept()

        return self.__class__(raw_socket), address

    async def bind(self, address: Union[Tuple[str, int], str]) -> None:
        await self._check_cancelled()
        if isinstance(address, tuple) and len(address) == 2:
            # For IP address/port combinations, call bind() directly
            try:
                ip_address(address[0])
            except ValueError:
                pass
            else:
                self._raw_socket.bind(address)
                return

        # In all other cases, do this in a worker thread to avoid blocking the event loop thread
        await self._run_in_thread(self._raw_socket.bind, address)

    async def connect(self, address: Union[tuple, str, bytes]) -> None:
        await self._check_cancelled()
        try:
            self._raw_socket.connect(address)
        except BlockingIOError:
            await self._wait_writable()

        error = self._raw_socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
        if error:
            raise OSError(error, errno.errorcode[error])

    async def recv(self, size: int, *, flags: int = 0) -> bytes:
        while True:
            await self._check_cancelled()
            try:
                return self._raw_socket.recv(size, flags)
            except (BlockingIOError, ssl.SSLWantReadError):
                await self._wait_readable()
            except ssl.SSLWantWriteError:
                await self._wait_writable()

    async def recv_into(self, buffer, nbytes: int, *, flags: int = 0) -> int:
        while True:
            await self._check_cancelled()
            try:
                return self._raw_socket.recv_into(buffer, nbytes, flags)
            except (BlockingIOError, ssl.SSLWantReadError):
                await self._wait_readable()
            except ssl.SSLWantWriteError:
                await self._wait_writable()

    async def recvfrom(self, size: int, *, flags: int = 0) -> Tuple[bytes, Any]:
        await self._check_cancelled()
        try:
            return self._raw_socket.recvfrom(size, flags)
        except BlockingIOError:
            await self._wait_readable()
            return self._raw_socket.recvfrom(size, flags)

    async def recvfrom_into(self, buffer, size: int, *, flags: int = 0):
        await self._check_cancelled()
        try:
            return self._raw_socket.recvfrom_into(buffer, size, flags)
        except BlockingIOError:
            await self._wait_readable()
            return self._raw_socket.recvfrom_into(buffer, size, flags)

    async def send(self, data: bytes, *, flags: int = 0) -> int:
        while True:
            await self._check_cancelled()
            try:
                return self._raw_socket.send(data, flags)
            except (BlockingIOError, ssl.SSLWantWriteError):
                await self._wait_writable()
            except ssl.SSLWantReadError:
                await self._wait_readable()

    async def sendto(self, data: bytes, addr, *, flags: int = 0) -> int:
        await self._check_cancelled()
        try:
            return self._raw_socket.sendto(data, flags, addr)
        except BlockingIOError:
            await self._wait_writable()
            return self._raw_socket.sendto(data, flags, addr)

    async def sendall(self, data: bytes, *, flags: int = 0) -> None:
        to_send = len(data)
        while to_send > 0:
            await self._check_cancelled()
            try:
                sent = self._raw_socket.send(data, flags)
            except (BlockingIOError, ssl.SSLWantWriteError):
                await self._wait_writable()
            except ssl.SSLWantReadError:
                await self._wait_readable()
            else:
                to_send -= sent

    async def start_tls(self, context: ssl.SSLContext,
                        server_hostname: Optional[str] = None) -> None:
        plain_socket = self._raw_socket
        self._raw_socket = context.wrap_socket(
            self._raw_socket, server_side=not server_hostname, do_handshake_on_connect=False,
            server_hostname=server_hostname)
        while True:
            try:
                self._raw_socket.do_handshake()
                return
            except ssl.SSLWantReadError:
                await self._wait_readable()
            except ssl.SSLWantWriteError:
                await self._wait_writable()
            except BaseException:
                self._raw_socket = plain_socket
                raise


class SocketStream(abc.SocketStream):
    __slots__ = '_socket', '_ssl_context', '_server_hostname'

    def __init__(self, sock: BaseSocket, ssl_context: Optional[ssl.SSLContext] = None,
                 server_hostname: Optional[str] = None) -> None:
        self._socket = sock
        self._ssl_context = ssl_context
        self._server_hostname = server_hostname

    def close(self):
        self._socket.close()

    async def receive_some(self, max_bytes: Optional[int]) -> bytes:
        return await self._socket.recv(max_bytes)

    async def receive_exactly(self, nbytes: int) -> bytes:
        buf = bytearray(nbytes)
        view = memoryview(buf)
        while nbytes > 0:
            bytes_read = await self._socket.recv_into(view, nbytes)
            if bytes_read == 0:
                total_bytes_read = len(buf) - nbytes
                raise IncompleteRead(buf[:total_bytes_read])

            view = view[bytes_read:]
            nbytes -= bytes_read

        return bytes(buf)

    async def receive_until(self, delimiter: bytes, max_size: int) -> bytes:
        offset = 0
        delimiter_size = len(delimiter)
        buf = b''
        while len(buf) < max_size:
            read_size = max_size - len(buf)
            data = await self._socket.recv(read_size, flags=socket.MSG_PEEK)
            if data == b'':
                raise IncompleteRead(buf)

            buf += data
            index = buf.find(delimiter, offset)
            if index >= 0:
                await self._socket.recv(index + 1)
                return buf[:index]
            else:
                await self._socket.recv(len(data))
                offset += len(data) - delimiter_size + 1

        raise DelimiterNotFound(buf)

    async def send_all(self, data: BufferType) -> None:
        return await self._socket.sendall(data)

    async def start_tls(self, context: Optional[ssl.SSLContext] = None) -> None:
        ssl_context = context or self._ssl_context or ssl.create_default_context()
        await self._socket.start_tls(ssl_context, self._server_hostname)


class SocketStreamServer(abc.SocketStreamServer):
    __slots__ = '_socket', '_ssl_context'

    def __init__(self, sock: BaseSocket, ssl_context: Optional[ssl.SSLContext]) -> None:
        self._socket = sock
        self._ssl_context = ssl_context

    def close(self) -> None:
        self._socket.close()

    @property
    def address(self) -> Union[tuple, str]:
        return self._socket.getsockname()

    async def accept(self):
        sock, addr = await self._socket.accept()
        try:
            stream = SocketStream(sock)
            if self._ssl_context:
                await stream.start_tls(self._ssl_context)

            return stream
        except BaseException:
            sock.close()
            raise


class DatagramSocket(abc.DatagramSocket):
    __slots__ = '_socket'

    def __init__(self, sock: BaseSocket) -> None:
        self._socket = sock

    def close(self):
        self._socket.close()

    @property
    def address(self) -> Union[Tuple[str, int], str]:
        return self._socket.getsockname()

    async def receive(self, max_bytes: int) -> Tuple[bytes, str]:
        return await self._socket.recvfrom(max_bytes)

    async def send(self, data: bytes, address: Optional[IPAddressType] = None,
                   port: Optional[int] = None) -> None:
        if address is not None and port is not None:
            await self._socket.sendto(data, (str(address), port))
        else:
            await self._socket.send(data)



from setuptools import setup

setup(
    use_scm_version={
        'version_scheme': 'post-release',
        'local_scheme': 'dirty-tag'
    },
    setup_requires=[
        'setuptools_scm >= 1.7.0'
    ]
)



from functools import partial
from inspect import iscoroutinefunction

import pytest

import anyio

try:
    from async_generator import isasyncgenfunction
except ImportError:
    from inspect import isasyncgenfunction


@pytest.hookimpl(hookwrapper=True)
def pytest_fixture_setup(fixturedef, request):
    def wrapper(*args, **kwargs):
        backend = kwargs['anyio_backend']
        if strip_backend:
            del kwargs['anyio_backend']

        if isasyncgenfunction(func):
            gen = func(*args, **kwargs)
            try:
                value = anyio.run(gen.__anext__, backend=backend)
            except StopAsyncIteration:
                raise RuntimeError('Async generator did not yield')

            yield value

            try:
                anyio.run(gen.__anext__, backend=backend)
            except StopAsyncIteration:
                pass
            else:
                anyio.run(gen.aclose)
                raise RuntimeError('Async generator fixture did not stop')
        else:
            yield anyio.run(partial(func, *args, **kwargs), backend=backend)

    func = fixturedef.func
    if isasyncgenfunction(func) or iscoroutinefunction(func):
        strip_backend = False
        if 'anyio_backend' not in fixturedef.argnames:
            fixturedef.argnames += ('anyio_backend',)
            strip_backend = True

        fixturedef.func = wrapper

    yield


def pytest_generate_tests(metafunc):
    marker = metafunc.definition.get_closest_marker('anyio')
    if marker:
        backends = marker.kwargs.get('backends', ['asyncio', 'curio', 'trio'])
        metafunc.fixturenames.append('anyio_backend')
        metafunc.parametrize('anyio_backend', backends, scope='session')


@pytest.mark.tryfirst
def pytest_pyfunc_call(pyfuncitem):
    if pyfuncitem.get_closest_marker('anyio'):
        funcargs = pyfuncitem.funcargs
        backend = pyfuncitem._request.getfixturevalue('anyio_backend')
        testargs = {arg: funcargs[arg] for arg in pyfuncitem._fixtureinfo.argnames}
        anyio.run(partial(pyfuncitem.obj, **testargs), backend=backend)
        return True
